/*
 *  Copyright (c) 2021 The WebRTC project authors. All Rights Reserved.
 *
 *  Use of this source code is governed by a BSD-style license
 *  that can be found in the LICENSE file in the root of the source
 *  tree. An additional intellectual property rights grant can be found
 *  in the file PATENTS.  All contributing project authors may
 *  be found in the AUTHORS file in the root of the source tree.
 */

#import "RTCIceCandidateErrorEvent.h"

#include <string>

NS_ASSUME_NONNULL_BEGIN

@interface RTC_OBJC_TYPE (RTCIceCandidateErrorEvent)
()

    - (instancetype)initWithAddress : (const std::string&)address port : (const int)port url
    : (const std::string&)url errorCode : (const int)errorCode errorText
    : (const std::string&)errorText;

@end

NS_ASSUME_NONNULL_END
